from enum import Enum
from typing import List, Optional, Dict

from pydantic import BaseModel, Field

from datagen_protocol.config import conf
from datagen_protocol.schema import fields, ExtrinsicParams
from datagen_protocol.schema.base import Asset, AssetAttributes, PresetAsset, SchemaBaseModel
from datagen_protocol.schema.attributes import (
    HICDomain,
    HICSubDomain,
    HumanBehaviour,
    Ethnicity,
    Age,
    Gender,
)
from datagen_protocol.schema.environment import Background, SequenceCamera, Light


class SequenceAttributes(AssetAttributes):
    domain: HICDomain
    sub_domain: HICSubDomain
    behaviour: HumanBehaviour
    ethnicity: Ethnicity
    gender: Gender
    age: Age


class SequencePresets(SchemaBaseModel):
    cameras: Dict[str, ExtrinsicParams]
    clutter_areas: List[str]


class ClutterLevel(str, Enum):
    NONE = "none"  # 0
    LOW = "low"  # 8
    MEDIUM = "medium"  # 16
    HIGH = "high"  # 32


class DataSequence(PresetAsset[SequenceAttributes, SequencePresets]):
    camera: SequenceCamera = Field(default_factory=SequenceCamera)
    background: Optional[Background] = None
    lights: Optional[List[Light]] = Field(default_factory=list)
    clutter_areas: Dict[str, ClutterLevel] = Field(default_factory=dict)



import sys

from pathlib import Path
from dynaconf import LazySettings


SCHEMAS_DEFAULTS = Path(__file__).parent.joinpath("resources", "defaults.toml")

SCHEMAS_DOCS = Path(__file__).parent.joinpath("resources", "documentation.toml")

conf = LazySettings(
    SETTINGS_FILE_FOR_DYNACONF=[str(SCHEMAS_DEFAULTS), str(SCHEMAS_DOCS)],
)



from enum import Enum
from typing import Optional
from pydantic.fields import Field

from datagen_protocol.schema.attributes import Gender, AccessoryPosition, GlassesStyle, MaskStyle
from datagen_protocol.schema.base import Asset, AttributesList, AssetAttributes, SchemaBaseModel
from datagen_protocol.config import conf
from datagen_protocol.schema import fields


class AccessoryAttributes(AssetAttributes):
    gender: AttributesList[Gender]
    supported_position: AttributesList[AccessoryPosition]


class GlassesAttributes(AccessoryAttributes):
    style: GlassesStyle


class LensColor(str, Enum):
    BLUE = "blue"
    GREEN = "green"
    YELLOW = "yellow"
    BLACK = "black"
    RED = "red"


class FrameColor(str, Enum):
    BLACK = "black"
    WHITE = "white"
    BLUE = "blue"
    RED = "red"
    GREEN = "green"
    GRAY = "gray"
    SILVER = "silver"
    GOLD = "gold"


class GlassesPosition(str, Enum):
    ON_NOSE = "on_nose"


class Glasses(Asset[GlassesAttributes]):
    lens_color: LensColor = fields.enum(LensColor, conf["accessories"]["glasses"]["lens"]["color"])
    lens_reflectivity: float = fields.numeric(conf["accessories"]["glasses"]["lens"]["reflectivity"])
    lens_transparency: float = fields.numeric(conf["accessories"]["glasses"]["lens"]["transparency"])
    frame_color: FrameColor = fields.enum(FrameColor, conf["accessories"]["glasses"]["frame"]["color"])
    frame_metalness: float = fields.numeric(conf["accessories"]["glasses"]["frame"]["metalness"])
    position: GlassesPosition = fields.enum(GlassesPosition, conf["accessories"]["glasses"]["position"])


class MaskColor(str, Enum):
    BLUE = "blue"
    GREEN = "green"
    YELLOW = "yellow"
    BLACK = "black"
    RED = "red"


class MaskAttributes(AccessoryAttributes):
    style: MaskStyle


class MaskPosition(str, Enum):
    ON_NOSE = "on_nose"
    ON_MOUTH = "on_mouth"
    ON_CHIN = "on_chin"


class MaskTexture(str, Enum):
    CLOTH = "cloth"
    DIAMOND_PATTERN = "diamond_pattern"
    WOVEN = "woven"


class Mask(Asset[MaskAttributes]):
    color: MaskColor = fields.enum(MaskColor, conf["accessories"]["mask"]["color"])
    texture: MaskTexture = fields.enum(MaskTexture, conf["accessories"]["mask"]["texture"])
    position: MaskPosition = fields.enum(MaskPosition, conf["accessories"]["mask"]["position"])
    roughness: float = fields.numeric(conf["accessories"]["mask"]["roughness"])


class Accessories(SchemaBaseModel):
    glasses: Optional[Glasses]
    mask: Optional[Mask]



from enum import Enum

from datagen_protocol.config import conf
from datagen_protocol.schema.base import SchemaBaseModel
from datagen_protocol.schema import fields
from datagen_protocol.schema.d3 import Point, Rotation


class LightType(str, Enum):
    NIR = "nir"


class Light(SchemaBaseModel):
    light_type: LightType
    beam_angle: float = fields.numeric(conf["light"]["beam_angle"])
    brightness: float = fields.numeric(conf["light"]["brightness"])
    falloff: float = fields.numeric(conf["light"]["falloff"])
    location: Point = fields.point(conf["light"]["location"])
    rotation: Rotation = fields.rotation(conf["light"]["rotation"])



import abc
from typing import TypeVar, Type

import numpy as np
from pydantic import BaseModel, Field

from datagen_protocol.schema import d3 as core_3d_schema


class Functional3dObject(abc.ABC):
    @abc.abstractmethod
    def to_array(self) -> np.ndarray:
        pass

    @classmethod
    @abc.abstractmethod
    def from_array(cls, arr: np.ndarray):
        pass


class Point(core_3d_schema.Point, Functional3dObject):
    def to_array(self) -> np.ndarray:
        return np.array([self.x, self.y, self.z])

    @classmethod
    def from_array(cls, arr: np.ndarray):
        return cls(x=arr[0], y=arr[1], z=arr[2])


class Vector(core_3d_schema.Vector, Functional3dObject):
    def to_array(self) -> np.ndarray:
        return np.array([self.x, self.y, self.z])

    @classmethod
    def from_array(cls, arr: np.ndarray):
        return cls(x=arr[0], y=arr[1], z=arr[2])


class Rotation(core_3d_schema.Rotation, Functional3dObject):
    def to_array(self) -> np.ndarray:
        return self._to_yaw_pitch_roll()

    def _to_yaw_pitch_roll(self) -> np.ndarray:
        return np.array([self.yaw, self.pitch, self.roll])

    @classmethod
    def from_array(cls, arr: np.ndarray):
        return cls._from_yaw_pitch_roll(arr)

    @classmethod
    def _from_yaw_pitch_roll(cls, arr: np.ndarray):
        return cls(yaw=arr[0], pitch=arr[1], roll=arr[2])


core_3d_schema.Point = Point
core_3d_schema.Rotation = Rotation
core_3d_schema.Vector = Vector



from datagen_protocol.schema.d3 import Point, Rotation, Vector
from datagen_protocol.schema.environment import (
    Background,
    Camera,
    ExtrinsicParams,
    IntrinsicParams,
    Light,
    LightType,
    Projection,
    Wavelength,
)
from datagen_protocol.schema.humans import (
    Accessories,
    FrameColor,
    Glasses,
    GlassesPosition,
    LensColor,
    Mask,
    MaskColor,
    MaskPosition,
    MaskTexture,
    Expression,
    ExpressionName,
    Eyes,
    Gaze,
    Hair,
    Eyebrows,
    FacialHair,
    HairColor,
    Head,
    Human,
    HumanDatapoint,
)
from datagen_protocol.schema.request import DataRequest
from datagen_protocol.schema.hic.sequence import ClutterLevel, DataSequence
